/************************************************************************
 *									*
 * (C) Copyright 2001 - Analog Devices, Inc.  All rights reserved.	*
 *									*
 * File Name: SPI_Slave.asm						*
 *									*
 * Date  : 6/22/01							*
 * Author: J.Beauchemin							*
 * Rev   : 0.0								*
 *									*
 * PURPOSE:	SPI Slave routine for the ADSP-2191.			*
 * 									*
 ************************************************************************/
#include <def2191.h>

/************************************************************************
 * GLOBAL & EXTERNAL DECLARATIONS					*
 ************************************************************************/ 
.GLOBAL Start;

/************************************************************************
 * DM data 								*
 ************************************************************************/
.section/data data1;
.var TX_Buf_SLAVE[16] = 0xFFFF, 0xEEEE, 0xDDDD, 0xCCCC, 0xBBBB, 0xAAAA, 0x9999, 0x8888, 0x7777, 0x6666, 0x5555, 0x4444, 0x3333, 0x2222, 0x1111, 0x0000;
.var RX_Buf_SLAVE[16];          /* Buffer For SPI0 Receive Data */

/************************************************************************  
 * Program memory code 							*
 ************************************************************************/
.SECTION /pm program;
Start:
_main:
	call Program_SPI0_Interrupt; /* Initialize Interrupt Priorities */	
	call SPI0_Register_Initialization;	     /* Initialize SPI0 */
	call Initiate_Transfers;		      /* Start Transfer */

wait_forever:
	jump wait_forever;

/************************************************************************
 * INTERRUPT PRIORITY CONFIGURATION					*
 ************************************************************************/ 
.section/code program;		// PROGRAM STARTS HERE
Program_SPI0_Interrupt:
  	IOPG = 0;
/* Line 47 */  	ar=io(SYSCR);		 /* Map Interrupt Vector Table to Page 0*/
  	ar = setbit 4 of ar;
	ar = setbit 0 of ar;	      /* Turn on SPI Ports (Not SPORT2) */
  	io(SYSCR)=ar;

	DIS int;			      /* Disable all interrupts */
  	IRPTL = 0x0;				/* Clear all interrupts */
  	ICNTL = 0x0;			   /* Interrupt nesting disable */
  	IMASK = 0;				 /* Mask all interrupts */
	
/* Line 57 - Setup Interrupt Priorities */
   	IOPG = Interrupt_Controller_Page;	
	ar = 0xBB1B;			   /* Assign SPI0 priority of 1 */
  	io(IPR1) = ar;
  	ar = 0xBBBB;	   /* Assign the remainder with lowest priority */
  	io(IPR0) = ar;
  	io(IPR2) = ar;						
	io(IPR3) = ar; 	

	AY0=IMASK;
/* 67 */	AY1=0x0020;				/* Unmask SPI Interrupt */
	AR = AY0 or AY1;						
	IMASK=AR;

	RTS;
/************************************************************************
 * SPI0 REGISTER INTIALIZATION	- SLAVE					*
 ************************************************************************/
.SECTION /pm program;
SPI0_Register_Initialization:

	IOPG = SPI0_Controller_Page;

	/* Setup SPI0 Configuration Reigster */
// Write to SPI Control Register
// TIMOD [1:0] = 00 : Transfer on RDBR Read. Interrupt When RDBR is Full
// SZ [2]      =  0 : Send Last Word When TDBR0 Is Empty
// GM [3]      =  1 : Discard Incoming Data If RDBR0 Is Full
// PSSE [4]    =  1 : Enables Slave-Select As Input (Slave)
// EMISO [5]   =  1 : MISO Enabled As Output (Slave)
// [7] and [6] =  0 : RESERVED
// SIZE [8]    =  1 : 16 Bit Word Length Select
// LSBF [9]    =  0 : Transmit MSB First
// CPHA [10]   =  1 : SCLK Starts Toggling At START Of First Data Bit
// CPOL [11]   =  1 : Active HIGH Serial Clock
// MSTR [12]   =  0 : Device Is Slave
// WOM [13]    =  0 : Normal MOSI/MISO Data Output (No Open Drain)
// SPE [14]    =  1 : SPI Module Is Enabled
// [15]        =  0 : RESERVED

/* 97 */	AR = 0x4D38;
	IO(SPICTL0) = AR;			/* Enable SPI0 as SLAVE */

	RTS;
/************************************************************************
 * INITIATE AND START TRANSFER						*
 ************************************************************************/
.SECTION /pm program;
Initiate_Transfers:
	/* Initialize DAG registers */
/* 107 */	I0 = TX_Buf_SLAVE;		   /* Pointer to TX SPI0 Buffer */
	I1 = RX_Buf_SLAVE;		   /* Pointer to RX SPI0 Buffer */
	M1 = 1;
	L0 = 0;					   /* Linear Addressing */
	L1 = 0;	
	
	ENA INT;			  /* Globally enable interrupts */

	IOPG = SPI0_Controller_Page;	    /* Get On The Right IO Page */

/* 117 */	CNTR = length(TX_Buf_SLAVE);   /* loop to Xmit entire TX buffer */

/* Loop until entire TX buffer has been recieved */
	DO looping until ce;
	AR = DM(I0, M1);	
	IO(TDBR0) = AR;		       /* Write to SPI0 Transmit Buffer */
looping:	idle;		     /* Wait for SPI0 Receive Interrupt */
	
	DIS INT;			 /* Globally disable Interrupts */
	RTS;